"""
Pydantic Models for CMDB - wireless_controller/vap

Runtime validation models for wireless_controller/vap configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class VapVlanPool(BaseModel):
    """
    Child table model for vlan-pool.
    
    VLAN pool.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4094, default=0, serialization_alias="id", description="ID.")    
    wtp_group: str | None = Field(max_length=35, default=None, description="WTP group name.")  # datasource: ['wireless-controller.wtp-group.name']
class VapVlanName(BaseModel):
    """
    Child table model for vlan-name.
    
    Table for mapping VLAN name to VLAN ID.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=35, description="VLAN name.")    
    vlan_id: list[int] = Field(ge=0, le=4094, default_factory=list, description="VLAN IDs (maximum 8 VLAN IDs).")
class VapUsergroup(BaseModel):
    """
    Child table model for usergroup.
    
    Firewall user group to be used to authenticate WiFi users.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="User group name.")  # datasource: ['user.group.name']
class VapSelectedUsergroups(BaseModel):
    """
    Child table model for selected-usergroups.
    
    Selective user groups that are permitted to authenticate.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="User group name.")  # datasource: ['user.group.name']
class VapSchedule(BaseModel):
    """
    Child table model for schedule.
    
    Firewall schedules for enabling this VAP on the FortiAP. This VAP will be enabled when at least one of the schedules is valid. Separate multiple schedule names with a space.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=35, description="Schedule name.")  # datasource: ['firewall.schedule.group.name', 'firewall.schedule.recurring.name', 'firewall.schedule.onetime.name']
class VapRadiusMacAuthUsergroups(BaseModel):
    """
    Child table model for radius-mac-auth-usergroups.
    
    Selective user groups that are permitted for RADIUS mac authentication.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str = Field(max_length=79, description="User group name.")  # datasource: ['user.group.name']
class VapPortalMessageOverrides(BaseModel):
    """
    Child table model for portal-message-overrides.
    
    Individual message overrides.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    auth_disclaimer_page: str | None = Field(max_length=35, default=None, description="Override auth-disclaimer-page message with message from portal-message-overrides group.")    
    auth_reject_page: str | None = Field(max_length=35, default=None, description="Override auth-reject-page message with message from portal-message-overrides group.")    
    auth_login_page: str | None = Field(max_length=35, default=None, description="Override auth-login-page message with message from portal-message-overrides group.")    
    auth_login_failed_page: str | None = Field(max_length=35, default=None, description="Override auth-login-failed-page message with message from portal-message-overrides group.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class VapSecurityEnum(str, Enum):
    """Allowed values for security field."""
    OPEN = "open"
    WEP64 = "wep64"
    WEP128 = "wep128"
    WPA_PERSONAL = "wpa-personal"
    WPA_ENTERPRISE = "wpa-enterprise"
    WPA_ONLY_PERSONAL = "wpa-only-personal"
    WPA_ONLY_ENTERPRISE = "wpa-only-enterprise"
    WPA2_ONLY_PERSONAL = "wpa2-only-personal"
    WPA2_ONLY_ENTERPRISE = "wpa2-only-enterprise"
    WPA3_ENTERPRISE = "wpa3-enterprise"
    WPA3_ONLY_ENTERPRISE = "wpa3-only-enterprise"
    WPA3_ENTERPRISE_TRANSITION = "wpa3-enterprise-transition"
    WPA3_SAE = "wpa3-sae"
    WPA3_SAE_TRANSITION = "wpa3-sae-transition"
    OWE = "owe"
    OSEN = "osen"

class VapMacUsernameDelimiterEnum(str, Enum):
    """Allowed values for mac_username_delimiter field."""
    HYPHEN = "hyphen"
    SINGLE_HYPHEN = "single-hyphen"
    COLON = "colon"
    NONE = "none"

class VapMacPasswordDelimiterEnum(str, Enum):
    """Allowed values for mac_password_delimiter field."""
    HYPHEN = "hyphen"
    SINGLE_HYPHEN = "single-hyphen"
    COLON = "colon"
    NONE = "none"

class VapMacCallingStationDelimiterEnum(str, Enum):
    """Allowed values for mac_calling_station_delimiter field."""
    HYPHEN = "hyphen"
    SINGLE_HYPHEN = "single-hyphen"
    COLON = "colon"
    NONE = "none"

class VapMacCalledStationDelimiterEnum(str, Enum):
    """Allowed values for mac_called_station_delimiter field."""
    HYPHEN = "hyphen"
    SINGLE_HYPHEN = "single-hyphen"
    COLON = "colon"
    NONE = "none"

class VapPortalTypeEnum(str, Enum):
    """Allowed values for portal_type field."""
    AUTH = "auth"
    AUTH_PLUSDISCLAIMER = "auth+disclaimer"
    DISCLAIMER = "disclaimer"
    EMAIL_COLLECT = "email-collect"
    CMCC = "cmcc"
    CMCC_MACAUTH = "cmcc-macauth"
    AUTH_MAC = "auth-mac"
    EXTERNAL_AUTH = "external-auth"
    EXTERNAL_MACAUTH = "external-macauth"

class VapLdpcEnum(str, Enum):
    """Allowed values for ldpc field."""
    DISABLE = "disable"
    RX = "rx"
    TX = "tx"
    RXTX = "rxtx"

class VapMulticastRateEnum(str, Enum):
    """Allowed values for multicast_rate field."""
    V_0 = "0"
    V_6000 = "6000"
    V_12000 = "12000"
    V_24000 = "24000"

class VapBroadcastSuppressionEnum(str, Enum):
    """Allowed values for broadcast_suppression field."""
    DHCP_UP = "dhcp-up"
    DHCP_DOWN = "dhcp-down"
    DHCP_STARVATION = "dhcp-starvation"
    DHCP_UCAST = "dhcp-ucast"
    ARP_KNOWN = "arp-known"
    ARP_UNKNOWN = "arp-unknown"
    ARP_REPLY = "arp-reply"
    ARP_POISON = "arp-poison"
    ARP_PROXY = "arp-proxy"
    NETBIOS_NS = "netbios-ns"
    NETBIOS_DS = "netbios-ds"
    IPV6 = "ipv6"
    ALL_OTHER_MC = "all-other-mc"
    ALL_OTHER_BC = "all-other-bc"

class VapIpv6RulesEnum(str, Enum):
    """Allowed values for ipv6_rules field."""
    DROP_ICMP6RA = "drop-icmp6ra"
    DROP_ICMP6RS = "drop-icmp6rs"
    DROP_LLMNR6 = "drop-llmnr6"
    DROP_ICMP6MLD2 = "drop-icmp6mld2"
    DROP_DHCP6S = "drop-dhcp6s"
    DROP_DHCP6C = "drop-dhcp6c"
    NDP_PROXY = "ndp-proxy"
    DROP_NS_DAD = "drop-ns-dad"
    DROP_NS_NONDAD = "drop-ns-nondad"

class VapVlanPoolingEnum(str, Enum):
    """Allowed values for vlan_pooling field."""
    WTP_GROUP = "wtp-group"
    ROUND_ROBIN = "round-robin"
    HASH = "hash"
    DISABLE = "disable"

class VapDhcpOption82CircuitIdInsertionEnum(str, Enum):
    """Allowed values for dhcp_option82_circuit_id_insertion field."""
    STYLE_1 = "style-1"
    STYLE_2 = "style-2"
    STYLE_3 = "style-3"
    DISABLE = "disable"

class VapRates11AEnum(str, Enum):
    """Allowed values for rates_11a field."""
    V_6 = "6"
    V_6_BASIC = "6-basic"
    V_9 = "9"
    V_9_BASIC = "9-basic"
    V_12 = "12"
    V_12_BASIC = "12-basic"
    V_18 = "18"
    V_18_BASIC = "18-basic"
    V_24 = "24"
    V_24_BASIC = "24-basic"
    V_36 = "36"
    V_36_BASIC = "36-basic"
    V_48 = "48"
    V_48_BASIC = "48-basic"
    V_54 = "54"
    V_54_BASIC = "54-basic"

class VapRates11BgEnum(str, Enum):
    """Allowed values for rates_11bg field."""
    V_1 = "1"
    V_1_BASIC = "1-basic"
    V_2 = "2"
    V_2_BASIC = "2-basic"
    V_5_5 = "5.5"
    V_5_5_BASIC = "5.5-basic"
    V_11 = "11"
    V_11_BASIC = "11-basic"
    V_6 = "6"
    V_6_BASIC = "6-basic"
    V_9 = "9"
    V_9_BASIC = "9-basic"
    V_12 = "12"
    V_12_BASIC = "12-basic"
    V_18 = "18"
    V_18_BASIC = "18-basic"
    V_24 = "24"
    V_24_BASIC = "24-basic"
    V_36 = "36"
    V_36_BASIC = "36-basic"
    V_48 = "48"
    V_48_BASIC = "48-basic"
    V_54 = "54"
    V_54_BASIC = "54-basic"

class VapRates11NSs12Enum(str, Enum):
    """Allowed values for rates_11n_ss12 field."""
    MCS01 = "mcs0/1"
    MCS11 = "mcs1/1"
    MCS21 = "mcs2/1"
    MCS31 = "mcs3/1"
    MCS41 = "mcs4/1"
    MCS51 = "mcs5/1"
    MCS61 = "mcs6/1"
    MCS71 = "mcs7/1"
    MCS82 = "mcs8/2"
    MCS92 = "mcs9/2"
    MCS102 = "mcs10/2"
    MCS112 = "mcs11/2"
    MCS122 = "mcs12/2"
    MCS132 = "mcs13/2"
    MCS142 = "mcs14/2"
    MCS152 = "mcs15/2"

class VapRates11NSs34Enum(str, Enum):
    """Allowed values for rates_11n_ss34 field."""
    MCS163 = "mcs16/3"
    MCS173 = "mcs17/3"
    MCS183 = "mcs18/3"
    MCS193 = "mcs19/3"
    MCS203 = "mcs20/3"
    MCS213 = "mcs21/3"
    MCS223 = "mcs22/3"
    MCS233 = "mcs23/3"
    MCS244 = "mcs24/4"
    MCS254 = "mcs25/4"
    MCS264 = "mcs26/4"
    MCS274 = "mcs27/4"
    MCS284 = "mcs28/4"
    MCS294 = "mcs29/4"
    MCS304 = "mcs30/4"
    MCS314 = "mcs31/4"


# ============================================================================
# Main Model
# ============================================================================

class VapModel(BaseModel):
    """
    Pydantic model for wireless_controller/vap configuration.
    
    Configure Virtual Access Points (VAPs).
    
    Validation Rules:        - name: max_length=15 pattern=        - pre_auth: pattern=        - external_pre_auth: pattern=        - mesh_backhaul: pattern=        - atf_weight: min=0 max=100 pattern=        - max_clients: min=0 max=4294967295 pattern=        - max_clients_ap: min=0 max=4294967295 pattern=        - ssid: max_length=32 pattern=        - broadcast_ssid: pattern=        - security: pattern=        - pmf: pattern=        - pmf_assoc_comeback_timeout: min=1 max=20 pattern=        - pmf_sa_query_retry_timeout: min=1 max=5 pattern=        - beacon_protection: pattern=        - okc: pattern=        - mbo: pattern=        - gas_comeback_delay: min=100 max=10000 pattern=        - gas_fragmentation_limit: min=512 max=4096 pattern=        - mbo_cell_data_conn_pref: pattern=        - _80211k: pattern=        - _80211v: pattern=        - neighbor_report_dual_band: pattern=        - fast_bss_transition: pattern=        - ft_mobility_domain: min=1 max=65535 pattern=        - ft_r0_key_lifetime: min=1 max=65535 pattern=        - ft_over_ds: pattern=        - sae_groups: pattern=        - owe_groups: pattern=        - owe_transition: pattern=        - owe_transition_ssid: max_length=32 pattern=        - additional_akms: pattern=        - eapol_key_retries: pattern=        - tkip_counter_measure: pattern=        - external_web: max_length=1023 pattern=        - external_web_format: pattern=        - external_logout: max_length=127 pattern=        - mac_username_delimiter: pattern=        - mac_password_delimiter: pattern=        - mac_calling_station_delimiter: pattern=        - mac_called_station_delimiter: pattern=        - mac_case: pattern=        - called_station_id_type: pattern=        - mac_auth_bypass: pattern=        - radius_mac_auth: pattern=        - radius_mac_auth_server: max_length=35 pattern=        - radius_mac_auth_block_interval: min=30 max=864000 pattern=        - radius_mac_mpsk_auth: pattern=        - radius_mac_mpsk_timeout: min=300 max=864000 pattern=        - radius_mac_auth_usergroups: pattern=        - auth: pattern=        - encrypt: pattern=        - keyindex: min=1 max=4 pattern=        - key: max_length=128 pattern=        - passphrase: max_length=128 pattern=        - sae_password: max_length=128 pattern=        - sae_h2e_only: pattern=        - sae_hnp_only: pattern=        - sae_pk: pattern=        - sae_private_key: max_length=359 pattern=        - akm24_only: pattern=        - radius_server: max_length=35 pattern=        - nas_filter_rule: pattern=        - domain_name_stripping: pattern=        - mlo: pattern=        - local_standalone: pattern=        - local_standalone_nat: pattern=        - ip: pattern=        - dhcp_lease_time: min=300 max=8640000 pattern=        - local_standalone_dns: pattern=        - local_standalone_dns_ip: pattern=        - local_lan_partition: pattern=        - local_bridging: pattern=        - local_lan: pattern=        - local_authentication: pattern=        - usergroup: pattern=        - captive_portal: pattern=        - captive_network_assistant_bypass: pattern=        - portal_message_override_group: max_length=35 pattern=        - portal_message_overrides: pattern=        - portal_type: pattern=        - selected_usergroups: pattern=        - security_exempt_list: max_length=35 pattern=        - security_redirect_url: max_length=1023 pattern=        - auth_cert: max_length=35 pattern=        - auth_portal_addr: max_length=63 pattern=        - intra_vap_privacy: pattern=        - schedule: pattern=        - ldpc: pattern=        - high_efficiency: pattern=        - target_wake_time: pattern=        - port_macauth: pattern=        - port_macauth_timeout: min=60 max=65535 pattern=        - port_macauth_reauth_timeout: min=120 max=65535 pattern=        - bss_color_partial: pattern=        - mpsk_profile: max_length=35 pattern=        - split_tunneling: pattern=        - nac: pattern=        - nac_profile: max_length=35 pattern=        - vlanid: min=0 max=4094 pattern=        - vlan_auto: pattern=        - dynamic_vlan: pattern=        - captive_portal_fw_accounting: pattern=        - captive_portal_ac_name: max_length=35 pattern=        - captive_portal_auth_timeout: min=0 max=864000 pattern=        - multicast_rate: pattern=        - multicast_enhance: pattern=        - igmp_snooping: pattern=        - dhcp_address_enforcement: pattern=        - broadcast_suppression: pattern=        - ipv6_rules: pattern=        - me_disable_thresh: min=2 max=256 pattern=        - mu_mimo: pattern=        - probe_resp_suppression: pattern=        - probe_resp_threshold: max_length=7 pattern=        - radio_sensitivity: pattern=        - quarantine: pattern=        - radio_5g_threshold: max_length=7 pattern=        - radio_2g_threshold: max_length=7 pattern=        - vlan_name: pattern=        - vlan_pooling: pattern=        - vlan_pool: pattern=        - dhcp_option43_insertion: pattern=        - dhcp_option82_insertion: pattern=        - dhcp_option82_circuit_id_insertion: pattern=        - dhcp_option82_remote_id_insertion: pattern=        - ptk_rekey: pattern=        - ptk_rekey_intv: min=600 max=864000 pattern=        - gtk_rekey: pattern=        - gtk_rekey_intv: min=600 max=864000 pattern=        - eap_reauth: pattern=        - eap_reauth_intv: min=1800 max=864000 pattern=        - roaming_acct_interim_update: pattern=        - qos_profile: max_length=35 pattern=        - hotspot20_profile: max_length=35 pattern=        - access_control_list: max_length=35 pattern=        - primary_wag_profile: max_length=35 pattern=        - secondary_wag_profile: max_length=35 pattern=        - tunnel_echo_interval: min=1 max=65535 pattern=        - tunnel_fallback_interval: min=0 max=65535 pattern=        - rates_11a: pattern=        - rates_11bg: pattern=        - rates_11n_ss12: pattern=        - rates_11n_ss34: pattern=        - rates_11ac_mcs_map: max_length=63 pattern=        - rates_11ax_mcs_map: max_length=63 pattern=        - rates_11be_mcs_map: max_length=15 pattern=        - rates_11be_mcs_map_160: max_length=15 pattern=        - rates_11be_mcs_map_320: max_length=15 pattern=        - utm_profile: max_length=35 pattern=        - utm_status: pattern=        - utm_log: pattern=        - ips_sensor: max_length=47 pattern=        - application_list: max_length=47 pattern=        - antivirus_profile: max_length=47 pattern=        - webfilter_profile: max_length=47 pattern=        - scan_botnet_connections: pattern=        - address_group: max_length=79 pattern=        - address_group_policy: pattern=        - sticky_client_remove: pattern=        - sticky_client_threshold_5g: max_length=7 pattern=        - sticky_client_threshold_2g: max_length=7 pattern=        - sticky_client_threshold_6g: max_length=7 pattern=        - bstm_rssi_disassoc_timer: min=1 max=2000 pattern=        - bstm_load_balancing_disassoc_timer: min=1 max=30 pattern=        - bstm_disassociation_imminent: pattern=        - beacon_advertising: pattern=        - osen: pattern=        - application_detection_engine: pattern=        - application_dscp_marking: pattern=        - application_report_intv: min=30 max=864000 pattern=        - l3_roaming: pattern=        - l3_roaming_mode: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=15, description="Virtual AP name.")    
    pre_auth: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable pre-authentication, where supported by clients (default = enable).")    
    external_pre_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable pre-authentication with external APs not managed by the FortiGate (default = disable).")    
    mesh_backhaul: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable using this VAP as a WiFi mesh backhaul (default = disable). This entry is only available when security is set to a WPA type or open.")    
    atf_weight: int | None = Field(ge=0, le=100, default=20, description="Airtime weight in percentage (default = 20).")    
    max_clients: int | None = Field(ge=0, le=4294967295, default=0, description="Maximum number of clients that can connect simultaneously to the VAP (default = 0, meaning no limitation).")    
    max_clients_ap: int | None = Field(ge=0, le=4294967295, default=0, description="Maximum number of clients that can connect simultaneously to the VAP per AP radio (default = 0, meaning no limitation).")    
    ssid: str | None = Field(max_length=32, default="fortinet", description="IEEE 802.11 service set identifier (SSID) for the wireless interface. Users who wish to use the wireless network must configure their computers to access this SSID name.")    
    broadcast_ssid: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable broadcasting the SSID (default = enable).")    
    security: VapSecurityEnum | None = Field(default=VapSecurityEnum.WPA2_ONLY_PERSONAL, description="Security mode for the wireless interface (default = wpa2-only-personal).")    
    pmf: Literal["disable", "enable", "optional"] | None = Field(default="disable", description="Protected Management Frames (PMF) support (default = disable).")    
    pmf_assoc_comeback_timeout: int | None = Field(ge=1, le=20, default=1, description="Protected Management Frames (PMF) comeback maximum timeout (1-20 sec).")    
    pmf_sa_query_retry_timeout: int | None = Field(ge=1, le=5, default=2, description="Protected Management Frames (PMF) SA query retry timeout interval (1 - 5 100s of msec).")    
    beacon_protection: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable beacon protection support (default = disable).")    
    okc: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable Opportunistic Key Caching (OKC) (default = enable).")    
    mbo: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable Multiband Operation (default = disable).")    
    gas_comeback_delay: int | None = Field(ge=100, le=10000, default=500, description="GAS comeback delay (0 or 100 - 10000 milliseconds, default = 500).")    
    gas_fragmentation_limit: int | None = Field(ge=512, le=4096, default=1024, description="GAS fragmentation limit (512 - 4096, default = 1024).")    
    mbo_cell_data_conn_pref: Literal["excluded", "prefer-not", "prefer-use"] | None = Field(default="prefer-not", description="MBO cell data connection preference (0, 1, or 255, default = 1).")    
    _80211k: Literal["disable", "enable"] | None = Field(default="enable", serialization_alias="80211k", description="Enable/disable 802.11k assisted roaming (default = enable).")    
    _80211v: Literal["disable", "enable"] | None = Field(default="enable", serialization_alias="80211v", description="Enable/disable 802.11v assisted roaming (default = enable).")    
    neighbor_report_dual_band: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable dual-band neighbor report (default = disable).")    
    fast_bss_transition: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable 802.11r Fast BSS Transition (FT) (default = disable).")    
    ft_mobility_domain: int | None = Field(ge=1, le=65535, default=1000, description="Mobility domain identifier in FT (1 - 65535, default = 1000).")    
    ft_r0_key_lifetime: int | None = Field(ge=1, le=65535, default=480, description="Lifetime of the PMK-R0 key in FT, 1-65535 minutes.")    
    ft_over_ds: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable FT over the Distribution System (DS).")    
    sae_groups: list[Literal["19", "20", "21"]] = Field(default_factory=list, description="SAE-Groups.")    
    owe_groups: list[Literal["19", "20", "21"]] = Field(default_factory=list, description="OWE-Groups.")    
    owe_transition: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable OWE transition mode support.")    
    owe_transition_ssid: str | None = Field(max_length=32, default=None, description="OWE transition mode peer SSID.")    
    additional_akms: list[Literal["akm6", "akm24"]] = Field(default_factory=list, description="Additional AKMs.")    
    eapol_key_retries: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable retransmission of EAPOL-Key frames (message 3/4 and group message 1/2) (default = enable).")    
    tkip_counter_measure: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable TKIP counter measure.")    
    external_web: str | None = Field(max_length=1023, default=None, description="URL of external authentication web server.")    
    external_web_format: Literal["auto-detect", "no-query-string", "partial-query-string"] | None = Field(default="auto-detect", description="URL query parameter detection (default = auto-detect).")    
    external_logout: str | None = Field(max_length=127, default=None, description="URL of external authentication logout server.")    
    mac_username_delimiter: VapMacUsernameDelimiterEnum | None = Field(default=VapMacUsernameDelimiterEnum.HYPHEN, description="MAC authentication username delimiter (default = hyphen).")    
    mac_password_delimiter: VapMacPasswordDelimiterEnum | None = Field(default=VapMacPasswordDelimiterEnum.HYPHEN, description="MAC authentication password delimiter (default = hyphen).")    
    mac_calling_station_delimiter: VapMacCallingStationDelimiterEnum | None = Field(default=VapMacCallingStationDelimiterEnum.HYPHEN, description="MAC calling station delimiter (default = hyphen).")    
    mac_called_station_delimiter: VapMacCalledStationDelimiterEnum | None = Field(default=VapMacCalledStationDelimiterEnum.HYPHEN, description="MAC called station delimiter (default = hyphen).")    
    mac_case: Literal["uppercase", "lowercase"] | None = Field(default="uppercase", description="MAC case (default = uppercase).")    
    called_station_id_type: Literal["mac", "ip", "apname"] | None = Field(default="mac", description="The format type of RADIUS attribute Called-Station-Id (default = mac).")    
    mac_auth_bypass: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable MAC authentication bypass.")    
    radius_mac_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable RADIUS-based MAC authentication of clients (default = disable).")    
    radius_mac_auth_server: str | None = Field(max_length=35, default=None, description="RADIUS-based MAC authentication server.")  # datasource: ['user.radius.name']    
    radius_mac_auth_block_interval: int | None = Field(ge=30, le=864000, default=0, description="Don't send RADIUS MAC auth request again if the client has been rejected within specific interval (0 or 30 - 864000 seconds, default = 0, 0 to disable blocking).")    
    radius_mac_mpsk_auth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable RADIUS-based MAC authentication of clients for MPSK authentication (default = disable).")    
    radius_mac_mpsk_timeout: int | None = Field(ge=300, le=864000, default=86400, description="RADIUS MAC MPSK cache timeout interval (0 or 300 - 864000, default = 86400, 0 to disable caching).")    
    radius_mac_auth_usergroups: list[VapRadiusMacAuthUsergroups] = Field(default_factory=list, description="Selective user groups that are permitted for RADIUS mac authentication.")    
    auth: Literal["radius", "usergroup"] | None = Field(default=None, description="Authentication protocol.")    
    encrypt: Literal["TKIP", "AES", "TKIP-AES"] | None = Field(default="AES", description="Encryption protocol to use (only available when security is set to a WPA type).")    
    keyindex: int | None = Field(ge=1, le=4, default=1, description="WEP key index (1 - 4).")    
    key: Any = Field(max_length=128, default=None, description="WEP Key.")    
    passphrase: Any = Field(max_length=128, default=None, description="WPA pre-shared key (PSK) to be used to authenticate WiFi users.")    
    sae_password: Any = Field(max_length=128, default=None, description="WPA3 SAE password to be used to authenticate WiFi users.")    
    sae_h2e_only: Literal["enable", "disable"] | None = Field(default="disable", description="Use hash-to-element-only mechanism for PWE derivation (default = disable).")    
    sae_hnp_only: Literal["enable", "disable"] | None = Field(default="disable", description="Use hunting-and-pecking-only mechanism for PWE derivation (default = disable).")    
    sae_pk: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable WPA3 SAE-PK (default = disable).")    
    sae_private_key: str | None = Field(max_length=359, default=None, description="Private key used for WPA3 SAE-PK authentication.")    
    akm24_only: Literal["disable", "enable"] | None = Field(default="disable", description="WPA3 SAE using group-dependent hash only (default = disable).")    
    radius_server: str | None = Field(max_length=35, default=None, description="RADIUS server to be used to authenticate WiFi users.")  # datasource: ['user.radius.name']    
    nas_filter_rule: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable NAS filter rule support (default = disable).")    
    domain_name_stripping: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable stripping domain name from identity (default = disable).")    
    mlo: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable WiFi7 Multi-Link-Operation (default = disable).")    
    local_standalone: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AP local standalone (default = disable).")    
    local_standalone_nat: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AP local standalone NAT mode.")    
    ip: Any = Field(default="0.0.0.0 0.0.0.0", description="IP address and subnet mask for the local standalone NAT subnet.")    
    dhcp_lease_time: int | None = Field(ge=300, le=8640000, default=2400, description="DHCP lease time in seconds for NAT IP address.")    
    local_standalone_dns: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AP local standalone DNS.")    
    local_standalone_dns_ip: list[str] = Field(default_factory=list, description="IPv4 addresses for the local standalone DNS.")    
    local_lan_partition: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable segregating client traffic to local LAN side (default = disable).")    
    local_bridging: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable bridging of wireless and Ethernet interfaces on the FortiAP (default = disable).")    
    local_lan: Literal["allow", "deny"] | None = Field(default="allow", description="Allow/deny traffic destined for a Class A, B, or C private IP address (default = allow).")    
    local_authentication: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable AP local authentication.")    
    usergroup: list[VapUsergroup] = Field(default_factory=list, description="Firewall user group to be used to authenticate WiFi users.")    
    captive_portal: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable captive portal.")    
    captive_network_assistant_bypass: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Captive Network Assistant bypass.")    
    portal_message_override_group: str | None = Field(max_length=35, default=None, description="Replacement message group for this VAP (only available when security is set to a captive portal type).")  # datasource: ['system.replacemsg-group.name']    
    portal_message_overrides: list[VapPortalMessageOverrides] = Field(default_factory=list, description="Individual message overrides.")    
    portal_type: VapPortalTypeEnum | None = Field(default=VapPortalTypeEnum.AUTH, description="Captive portal functionality. Configure how the captive portal authenticates users and whether it includes a disclaimer.")    
    selected_usergroups: list[VapSelectedUsergroups] = Field(default_factory=list, description="Selective user groups that are permitted to authenticate.")    
    security_exempt_list: str | None = Field(max_length=35, default=None, description="Optional security exempt list for captive portal authentication.")  # datasource: ['user.security-exempt-list.name']    
    security_redirect_url: str | None = Field(max_length=1023, default=None, description="Optional URL for redirecting users after they pass captive portal authentication.")    
    auth_cert: str | None = Field(max_length=35, default=None, description="HTTPS server certificate.")  # datasource: ['vpn.certificate.local.name']    
    auth_portal_addr: str | None = Field(max_length=63, default=None, description="Address of captive portal.")    
    intra_vap_privacy: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable blocking communication between clients on the same SSID (called intra-SSID privacy) (default = disable).")    
    schedule: list[VapSchedule] = Field(default_factory=list, description="Firewall schedules for enabling this VAP on the FortiAP. This VAP will be enabled when at least one of the schedules is valid. Separate multiple schedule names with a space.")    
    ldpc: VapLdpcEnum | None = Field(default=VapLdpcEnum.RXTX, description="VAP low-density parity-check (LDPC) coding configuration.")    
    high_efficiency: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable 802.11ax high efficiency (default = enable).")    
    target_wake_time: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable 802.11ax target wake time (default = enable).")    
    port_macauth: Literal["disable", "radius", "address-group"] | None = Field(default="disable", description="Enable/disable LAN port MAC authentication (default = disable).")    
    port_macauth_timeout: int | None = Field(ge=60, le=65535, default=600, description="LAN port MAC authentication idle timeout value (default = 600 sec).")    
    port_macauth_reauth_timeout: int | None = Field(ge=120, le=65535, default=7200, description="LAN port MAC authentication re-authentication timeout value (default = 7200 sec).")    
    bss_color_partial: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable 802.11ax partial BSS color (default = enable).")    
    mpsk_profile: str | None = Field(max_length=35, default=None, description="MPSK profile name.")  # datasource: ['wireless-controller.mpsk-profile.name']    
    split_tunneling: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable split tunneling (default = disable).")    
    nac: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable network access control.")    
    nac_profile: str | None = Field(max_length=35, default=None, description="NAC profile name.")  # datasource: ['wireless-controller.nac-profile.name']    
    vlanid: int | None = Field(ge=0, le=4094, default=0, description="Optional VLAN ID.")    
    vlan_auto: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable automatic management of SSID VLAN interface.")    
    dynamic_vlan: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable dynamic VLAN assignment.")    
    captive_portal_fw_accounting: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable RADIUS accounting for captive portal firewall authentication session.")    
    captive_portal_ac_name: str | None = Field(max_length=35, default=None, description="Local-bridging captive portal ac-name.")    
    captive_portal_auth_timeout: int | None = Field(ge=0, le=864000, default=0, description="Hard timeout - AP will always clear the session after timeout regardless of traffic (0 - 864000 sec, default = 0).")    
    multicast_rate: VapMulticastRateEnum | None = Field(default=VapMulticastRateEnum.V_0, description="Multicast rate (0, 6000, 12000, or 24000 kbps, default = 0).")    
    multicast_enhance: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable converting multicast to unicast to improve performance (default = disable).")    
    igmp_snooping: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable IGMP snooping.")    
    dhcp_address_enforcement: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable DHCP address enforcement (default = disable).")    
    broadcast_suppression: list[VapBroadcastSuppressionEnum] = Field(default_factory=list, description="Optional suppression of broadcast messages. For example, you can keep DHCP messages, ARP broadcasts, and so on off of the wireless network.")    
    ipv6_rules: list[VapIpv6RulesEnum] = Field(default_factory=list, description="Optional rules of IPv6 packets. For example, you can keep RA, RS and so on off of the wireless network.")    
    me_disable_thresh: int | None = Field(ge=2, le=256, default=32, description="Disable multicast enhancement when this many clients are receiving multicast traffic.")    
    mu_mimo: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable Multi-user MIMO (default = enable).")    
    probe_resp_suppression: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable probe response suppression (to ignore weak signals) (default = disable).")    
    probe_resp_threshold: str | None = Field(max_length=7, default="-80", description="Minimum signal level/threshold in dBm required for the AP response to probe requests (-95 to -20, default = -80).")    
    radio_sensitivity: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable software radio sensitivity (to ignore weak signals) (default = disable).")    
    quarantine: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable station quarantine (default = disable).")    
    radio_5g_threshold: str | None = Field(max_length=7, default="-76", description="Minimum signal level/threshold in dBm required for the AP response to receive a packet in 5G band(-95 to -20, default = -76).")    
    radio_2g_threshold: str | None = Field(max_length=7, default="-79", description="Minimum signal level/threshold in dBm required for the AP response to receive a packet in 2.4G band (-95 to -20, default = -79).")    
    vlan_name: list[VapVlanName] = Field(default_factory=list, description="Table for mapping VLAN name to VLAN ID.")    
    vlan_pooling: VapVlanPoolingEnum | None = Field(default=VapVlanPoolingEnum.DISABLE, description="Enable/disable VLAN pooling, to allow grouping of multiple wireless controller VLANs into VLAN pools (default = disable). When set to wtp-group, VLAN pooling occurs with VLAN assignment by wtp-group.")    
    vlan_pool: list[VapVlanPool] = Field(default_factory=list, description="VLAN pool.")    
    dhcp_option43_insertion: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable insertion of DHCP option 43 (default = enable).")    
    dhcp_option82_insertion: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable DHCP option 82 insert (default = disable).")    
    dhcp_option82_circuit_id_insertion: VapDhcpOption82CircuitIdInsertionEnum | None = Field(default=VapDhcpOption82CircuitIdInsertionEnum.DISABLE, description="Enable/disable DHCP option 82 circuit-id insert (default = disable).")    
    dhcp_option82_remote_id_insertion: Literal["style-1", "disable"] | None = Field(default="disable", description="Enable/disable DHCP option 82 remote-id insert (default = disable).")    
    ptk_rekey: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable PTK rekey for WPA-Enterprise security.")    
    ptk_rekey_intv: int | None = Field(ge=600, le=864000, default=86400, description="PTK rekey interval (600 - 864000 sec, default = 86400).")    
    gtk_rekey: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable GTK rekey for WPA security.")    
    gtk_rekey_intv: int | None = Field(ge=600, le=864000, default=86400, description="GTK rekey interval (600 - 864000 sec, default = 86400).")    
    eap_reauth: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable EAP re-authentication for WPA-Enterprise security.")    
    eap_reauth_intv: int | None = Field(ge=1800, le=864000, default=86400, description="EAP re-authentication interval (1800 - 864000 sec, default = 86400).")    
    roaming_acct_interim_update: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable using accounting interim update instead of accounting start/stop on roaming for WPA-Enterprise security.")    
    qos_profile: str | None = Field(max_length=35, default=None, description="Quality of service profile name.")  # datasource: ['wireless-controller.qos-profile.name']    
    hotspot20_profile: str | None = Field(max_length=35, default=None, description="Hotspot 2.0 profile name.")  # datasource: ['wireless-controller.hotspot20.hs-profile.name']    
    access_control_list: str | None = Field(max_length=35, default=None, description="Profile name for access-control-list.")  # datasource: ['wireless-controller.access-control-list.name']    
    primary_wag_profile: str | None = Field(max_length=35, default=None, description="Primary wireless access gateway profile name.")  # datasource: ['wireless-controller.wag-profile.name']    
    secondary_wag_profile: str | None = Field(max_length=35, default=None, description="Secondary wireless access gateway profile name.")  # datasource: ['wireless-controller.wag-profile.name']    
    tunnel_echo_interval: int | None = Field(ge=1, le=65535, default=300, description="The time interval to send echo to both primary and secondary tunnel peers (1 - 65535 sec, default = 300).")    
    tunnel_fallback_interval: int | None = Field(ge=0, le=65535, default=7200, description="The time interval for secondary tunnel to fall back to primary tunnel (0 - 65535 sec, default = 7200).")    
    rates_11a: list[VapRates11AEnum] = Field(default_factory=list, description="Allowed data rates for 802.11a.")    
    rates_11bg: list[VapRates11BgEnum] = Field(default_factory=list, description="Allowed data rates for 802.11b/g.")    
    rates_11n_ss12: list[VapRates11NSs12Enum] = Field(default_factory=list, description="Allowed data rates for 802.11n with 1 or 2 spatial streams.")    
    rates_11n_ss34: list[VapRates11NSs34Enum] = Field(default_factory=list, description="Allowed data rates for 802.11n with 3 or 4 spatial streams.")    
    rates_11ac_mcs_map: str | None = Field(max_length=63, default=None, description="Comma separated list of max supported VHT MCS for spatial streams 1 through 8.")    
    rates_11ax_mcs_map: str | None = Field(max_length=63, default=None, description="Comma separated list of max supported HE MCS for spatial streams 1 through 8.")    
    rates_11be_mcs_map: str | None = Field(max_length=15, default=None, description="Comma separated list of max nss that supports EHT-MCS 0-9, 10-11, 12-13 for 20MHz/40MHz/80MHz bandwidth.")    
    rates_11be_mcs_map_160: str | None = Field(max_length=15, default=None, description="Comma separated list of max nss that supports EHT-MCS 0-9, 10-11, 12-13 for 160MHz bandwidth.")    
    rates_11be_mcs_map_320: str | None = Field(max_length=15, default=None, description="Comma separated list of max nss that supports EHT-MCS 0-9, 10-11, 12-13 for 320MHz bandwidth.")    
    utm_profile: str | None = Field(max_length=35, default=None, description="UTM profile name.")  # datasource: ['wireless-controller.utm-profile.name']    
    utm_status: Literal["enable", "disable"] | None = Field(default="disable", description="Enable to add one or more security profiles (AV, IPS, etc.) to the VAP.")    
    utm_log: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable UTM logging.")    
    ips_sensor: str | None = Field(max_length=47, default=None, description="IPS sensor name.")  # datasource: ['ips.sensor.name']    
    application_list: str | None = Field(max_length=47, default=None, description="Application control list name.")  # datasource: ['application.list.name']    
    antivirus_profile: str | None = Field(max_length=47, default=None, description="AntiVirus profile name.")  # datasource: ['antivirus.profile.name']    
    webfilter_profile: str | None = Field(max_length=47, default=None, description="WebFilter profile name.")  # datasource: ['webfilter.profile.name']    
    scan_botnet_connections: Literal["disable", "monitor", "block"] | None = Field(default="monitor", description="Block or monitor connections to Botnet servers or disable Botnet scanning.")    
    address_group: str | None = Field(max_length=79, default=None, description="Firewall Address Group Name.")  # datasource: ['firewall.addrgrp.name']    
    address_group_policy: Literal["disable", "allow", "deny"] | None = Field(default="disable", description="Configure MAC address filtering policy for MAC addresses that are in the address-group.")    
    sticky_client_remove: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable sticky client remove to maintain good signal level clients in SSID (default = disable).")    
    sticky_client_threshold_5g: str | None = Field(max_length=7, default="-76", description="Minimum signal level/threshold in dBm required for the 5G client to be serviced by the AP (-95 to -20, default = -76).")    
    sticky_client_threshold_2g: str | None = Field(max_length=7, default="-79", description="Minimum signal level/threshold in dBm required for the 2G client to be serviced by the AP (-95 to -20, default = -79).")    
    sticky_client_threshold_6g: str | None = Field(max_length=7, default="-76", description="Minimum signal level/threshold in dBm required for the 6G client to be serviced by the AP (-95 to -20, default = -76).")    
    bstm_rssi_disassoc_timer: int | None = Field(ge=1, le=2000, default=200, description="Time interval for client to voluntarily leave AP before forcing a disassociation due to low RSSI (0 to 2000, default = 200).")    
    bstm_load_balancing_disassoc_timer: int | None = Field(ge=1, le=30, default=10, description="Time interval for client to voluntarily leave AP before forcing a disassociation due to AP load-balancing (0 to 30, default = 10).")    
    bstm_disassociation_imminent: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable forcing of disassociation after the BSTM request timer has been reached (default = enable).")    
    beacon_advertising: list[Literal["name", "model", "serial-number"]] = Field(default_factory=list, description="Fortinet beacon advertising IE data   (default = empty).")    
    osen: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable OSEN as part of key management (default = disable).")    
    application_detection_engine: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable application detection engine (default = disable).")    
    application_dscp_marking: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable application attribute based DSCP marking (default = disable).")    
    application_report_intv: int | None = Field(ge=30, le=864000, default=120, description="Application report interval (30 - 864000 sec, default = 120).")    
    l3_roaming: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable layer 3 roaming (default = disable).")    
    l3_roaming_mode: Literal["direct", "indirect"] | None = Field(default="direct", description="Select the way that layer 3 roaming traffic is passed (default = direct).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('radius_mac_auth_server')
    @classmethod
    def validate_radius_mac_auth_server(cls, v: Any) -> Any:
        """
        Validate radius_mac_auth_server field.
        
        Datasource: ['user.radius.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('radius_server')
    @classmethod
    def validate_radius_server(cls, v: Any) -> Any:
        """
        Validate radius_server field.
        
        Datasource: ['user.radius.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('portal_message_override_group')
    @classmethod
    def validate_portal_message_override_group(cls, v: Any) -> Any:
        """
        Validate portal_message_override_group field.
        
        Datasource: ['system.replacemsg-group.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('security_exempt_list')
    @classmethod
    def validate_security_exempt_list(cls, v: Any) -> Any:
        """
        Validate security_exempt_list field.
        
        Datasource: ['user.security-exempt-list.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('auth_cert')
    @classmethod
    def validate_auth_cert(cls, v: Any) -> Any:
        """
        Validate auth_cert field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('mpsk_profile')
    @classmethod
    def validate_mpsk_profile(cls, v: Any) -> Any:
        """
        Validate mpsk_profile field.
        
        Datasource: ['wireless-controller.mpsk-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('nac_profile')
    @classmethod
    def validate_nac_profile(cls, v: Any) -> Any:
        """
        Validate nac_profile field.
        
        Datasource: ['wireless-controller.nac-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('qos_profile')
    @classmethod
    def validate_qos_profile(cls, v: Any) -> Any:
        """
        Validate qos_profile field.
        
        Datasource: ['wireless-controller.qos-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('hotspot20_profile')
    @classmethod
    def validate_hotspot20_profile(cls, v: Any) -> Any:
        """
        Validate hotspot20_profile field.
        
        Datasource: ['wireless-controller.hotspot20.hs-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('access_control_list')
    @classmethod
    def validate_access_control_list(cls, v: Any) -> Any:
        """
        Validate access_control_list field.
        
        Datasource: ['wireless-controller.access-control-list.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('primary_wag_profile')
    @classmethod
    def validate_primary_wag_profile(cls, v: Any) -> Any:
        """
        Validate primary_wag_profile field.
        
        Datasource: ['wireless-controller.wag-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('secondary_wag_profile')
    @classmethod
    def validate_secondary_wag_profile(cls, v: Any) -> Any:
        """
        Validate secondary_wag_profile field.
        
        Datasource: ['wireless-controller.wag-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('utm_profile')
    @classmethod
    def validate_utm_profile(cls, v: Any) -> Any:
        """
        Validate utm_profile field.
        
        Datasource: ['wireless-controller.utm-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ips_sensor')
    @classmethod
    def validate_ips_sensor(cls, v: Any) -> Any:
        """
        Validate ips_sensor field.
        
        Datasource: ['ips.sensor.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('application_list')
    @classmethod
    def validate_application_list(cls, v: Any) -> Any:
        """
        Validate application_list field.
        
        Datasource: ['application.list.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('antivirus_profile')
    @classmethod
    def validate_antivirus_profile(cls, v: Any) -> Any:
        """
        Validate antivirus_profile field.
        
        Datasource: ['antivirus.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('webfilter_profile')
    @classmethod
    def validate_webfilter_profile(cls, v: Any) -> Any:
        """
        Validate webfilter_profile field.
        
        Datasource: ['webfilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('address_group')
    @classmethod
    def validate_address_group(cls, v: Any) -> Any:
        """
        Validate address_group field.
        
        Datasource: ['firewall.addrgrp.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "VapModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_radius_mac_auth_server_references(self, client: Any) -> list[str]:
        """
        Validate radius_mac_auth_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/radius        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     radius_mac_auth_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_radius_mac_auth_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "radius_mac_auth_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.user.radius.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Radius-Mac-Auth-Server '{value}' not found in "
                "user/radius"
            )        
        return errors    
    async def validate_radius_mac_auth_usergroups_references(self, client: Any) -> list[str]:
        """
        Validate radius_mac_auth_usergroups references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     radius_mac_auth_usergroups=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_radius_mac_auth_usergroups_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "radius_mac_auth_usergroups", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.user.group.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Radius-Mac-Auth-Usergroups '{value}' not found in "
                    "user/group"
                )        
        return errors    
    async def validate_radius_server_references(self, client: Any) -> list[str]:
        """
        Validate radius_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/radius        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     radius_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_radius_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "radius_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.user.radius.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Radius-Server '{value}' not found in "
                "user/radius"
            )        
        return errors    
    async def validate_usergroup_references(self, client: Any) -> list[str]:
        """
        Validate usergroup references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     usergroup=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_usergroup_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "usergroup", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.user.group.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Usergroup '{value}' not found in "
                    "user/group"
                )        
        return errors    
    async def validate_portal_message_override_group_references(self, client: Any) -> list[str]:
        """
        Validate portal_message_override_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/replacemsg-group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     portal_message_override_group="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_portal_message_override_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "portal_message_override_group", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.replacemsg_group.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Portal-Message-Override-Group '{value}' not found in "
                "system/replacemsg-group"
            )        
        return errors    
    async def validate_selected_usergroups_references(self, client: Any) -> list[str]:
        """
        Validate selected_usergroups references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     selected_usergroups=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_selected_usergroups_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "selected_usergroups", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.user.group.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Selected-Usergroups '{value}' not found in "
                    "user/group"
                )        
        return errors    
    async def validate_security_exempt_list_references(self, client: Any) -> list[str]:
        """
        Validate security_exempt_list references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - user/security-exempt-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     security_exempt_list="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_security_exempt_list_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "security_exempt_list", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.user.security_exempt_list.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Security-Exempt-List '{value}' not found in "
                "user/security-exempt-list"
            )        
        return errors    
    async def validate_auth_cert_references(self, client: Any) -> list[str]:
        """
        Validate auth_cert references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     auth_cert="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_auth_cert_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "auth_cert", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Auth-Cert '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_schedule_references(self, client: Any) -> list[str]:
        """
        Validate schedule references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/schedule/group        - firewall/schedule/recurring        - firewall/schedule/onetime        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     schedule=[{"name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_schedule_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "schedule", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("name")
            else:
                value = getattr(item, "name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.firewall.schedule.group.exists(value):
                found = True
            elif await client.api.cmdb.firewall.schedule.recurring.exists(value):
                found = True
            elif await client.api.cmdb.firewall.schedule.onetime.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Schedule '{value}' not found in "
                    "firewall/schedule/group or firewall/schedule/recurring or firewall/schedule/onetime"
                )        
        return errors    
    async def validate_mpsk_profile_references(self, client: Any) -> list[str]:
        """
        Validate mpsk_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/mpsk-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     mpsk_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_mpsk_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "mpsk_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.mpsk_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Mpsk-Profile '{value}' not found in "
                "wireless-controller/mpsk-profile"
            )        
        return errors    
    async def validate_nac_profile_references(self, client: Any) -> list[str]:
        """
        Validate nac_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/nac-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     nac_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_nac_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "nac_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.nac_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Nac-Profile '{value}' not found in "
                "wireless-controller/nac-profile"
            )        
        return errors    
    async def validate_vlan_pool_references(self, client: Any) -> list[str]:
        """
        Validate vlan_pool references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/wtp-group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     vlan_pool=[{"wtp-group": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_vlan_pool_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "vlan_pool", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("wtp-group")
            else:
                value = getattr(item, "wtp-group", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.wireless_controller.wtp_group.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Vlan-Pool '{value}' not found in "
                    "wireless-controller/wtp-group"
                )        
        return errors    
    async def validate_qos_profile_references(self, client: Any) -> list[str]:
        """
        Validate qos_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/qos-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     qos_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_qos_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "qos_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.qos_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Qos-Profile '{value}' not found in "
                "wireless-controller/qos-profile"
            )        
        return errors    
    async def validate_hotspot20_profile_references(self, client: Any) -> list[str]:
        """
        Validate hotspot20_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/hotspot20/hs-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     hotspot20_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_hotspot20_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "hotspot20_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.hotspot20.hs_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Hotspot20-Profile '{value}' not found in "
                "wireless-controller/hotspot20/hs-profile"
            )        
        return errors    
    async def validate_access_control_list_references(self, client: Any) -> list[str]:
        """
        Validate access_control_list references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/access-control-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     access_control_list="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_access_control_list_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "access_control_list", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.access_control_list.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Access-Control-List '{value}' not found in "
                "wireless-controller/access-control-list"
            )        
        return errors    
    async def validate_primary_wag_profile_references(self, client: Any) -> list[str]:
        """
        Validate primary_wag_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/wag-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     primary_wag_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_primary_wag_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "primary_wag_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.wag_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Primary-Wag-Profile '{value}' not found in "
                "wireless-controller/wag-profile"
            )        
        return errors    
    async def validate_secondary_wag_profile_references(self, client: Any) -> list[str]:
        """
        Validate secondary_wag_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/wag-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     secondary_wag_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_secondary_wag_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "secondary_wag_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.wag_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Secondary-Wag-Profile '{value}' not found in "
                "wireless-controller/wag-profile"
            )        
        return errors    
    async def validate_utm_profile_references(self, client: Any) -> list[str]:
        """
        Validate utm_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - wireless-controller/utm-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     utm_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_utm_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "utm_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.wireless_controller.utm_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Utm-Profile '{value}' not found in "
                "wireless-controller/utm-profile"
            )        
        return errors    
    async def validate_ips_sensor_references(self, client: Any) -> list[str]:
        """
        Validate ips_sensor references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - ips/sensor        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     ips_sensor="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ips_sensor_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ips_sensor", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.ips.sensor.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ips-Sensor '{value}' not found in "
                "ips/sensor"
            )        
        return errors    
    async def validate_application_list_references(self, client: Any) -> list[str]:
        """
        Validate application_list references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - application/list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     application_list="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_application_list_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "application_list", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.application.list.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Application-List '{value}' not found in "
                "application/list"
            )        
        return errors    
    async def validate_antivirus_profile_references(self, client: Any) -> list[str]:
        """
        Validate antivirus_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - antivirus/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     antivirus_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_antivirus_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "antivirus_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.antivirus.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Antivirus-Profile '{value}' not found in "
                "antivirus/profile"
            )        
        return errors    
    async def validate_webfilter_profile_references(self, client: Any) -> list[str]:
        """
        Validate webfilter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - webfilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     webfilter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_webfilter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "webfilter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.webfilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Webfilter-Profile '{value}' not found in "
                "webfilter/profile"
            )        
        return errors    
    async def validate_address_group_references(self, client: Any) -> list[str]:
        """
        Validate address_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/addrgrp        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VapModel(
            ...     address_group="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_address_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.wireless_controller.vap.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "address_group", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.addrgrp.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Address-Group '{value}' not found in "
                "firewall/addrgrp"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_radius_mac_auth_server_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_radius_mac_auth_usergroups_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_radius_server_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_usergroup_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_portal_message_override_group_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_selected_usergroups_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_security_exempt_list_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_auth_cert_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_schedule_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_mpsk_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_nac_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_vlan_pool_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_qos_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_hotspot20_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_access_control_list_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_primary_wag_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_secondary_wag_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_utm_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ips_sensor_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_application_list_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_antivirus_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_webfilter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_address_group_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "VapModel",    "VapRadiusMacAuthUsergroups",    "VapUsergroup",    "VapPortalMessageOverrides",    "VapSelectedUsergroups",    "VapSchedule",    "VapVlanName",    "VapVlanPool",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.048894Z
# ============================================================================


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: wireless_controller/setting
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class SettingOffendingssidItem:
    """Nested item for offending-ssid field - supports attribute access."""
    id: int
    ssid_pattern: str
    action: Literal["log", "suppress"]


class SettingDarrpoptimizeschedulesItem:
    """Nested item for darrp-optimize-schedules field - supports attribute access."""
    name: str


class SettingPayload(TypedDict, total=False):
    """Payload type for Setting operations."""
    account_id: str
    country: Literal["--", "AF", "AL", "DZ", "AS", "AO", "AR", "AM", "AU", "AT", "AZ", "BS", "BH", "BD", "BB", "BY", "BE", "BZ", "BJ", "BM", "BT", "BO", "BA", "BW", "BR", "BN", "BG", "BF", "KH", "CM", "KY", "CF", "TD", "CL", "CN", "CX", "CO", "CG", "CD", "CR", "HR", "CY", "CZ", "DK", "DJ", "DM", "DO", "EC", "EG", "SV", "ET", "EE", "GF", "PF", "FO", "FJ", "FI", "FR", "GA", "GE", "GM", "DE", "GH", "GI", "GR", "GL", "GD", "GP", "GU", "GT", "GY", "HT", "HN", "HK", "HU", "IS", "IN", "ID", "IQ", "IE", "IM", "IL", "IT", "CI", "JM", "JO", "KZ", "KE", "KR", "KW", "LA", "LV", "LB", "LS", "LR", "LY", "LI", "LT", "LU", "MO", "MK", "MG", "MW", "MY", "MV", "ML", "MT", "MH", "MQ", "MR", "MU", "YT", "MX", "FM", "MD", "MC", "MN", "MA", "MZ", "MM", "NA", "NP", "NL", "AN", "AW", "NZ", "NI", "NE", "NG", "NO", "MP", "OM", "PK", "PW", "PA", "PG", "PY", "PE", "PH", "PL", "PT", "PR", "QA", "RE", "RO", "RU", "RW", "BL", "KN", "LC", "MF", "PM", "VC", "SA", "SN", "RS", "ME", "SL", "SG", "SK", "SI", "SO", "ZA", "ES", "LK", "SR", "SZ", "SE", "CH", "TW", "TZ", "TH", "TL", "TG", "TT", "TN", "TR", "TM", "AE", "TC", "UG", "UA", "GB", "US", "PS", "UY", "UZ", "VU", "VE", "VN", "VI", "WF", "YE", "ZM", "ZW", "JP", "CA"]
    duplicate_ssid: Literal["enable", "disable"]
    fapc_compatibility: Literal["enable", "disable"]
    wfa_compatibility: Literal["enable", "disable"]
    phishing_ssid_detect: Literal["enable", "disable"]
    fake_ssid_action: str | list[str]
    offending_ssid: str | list[str] | list[dict[str, Any]] | list[SettingOffendingssidItem]
    device_weight: int
    device_holdoff: int
    device_idle: int
    firmware_provision_on_authorization: Literal["enable", "disable"]
    rolling_wtp_upgrade: Literal["enable", "disable"]
    darrp_optimize: int
    darrp_optimize_schedules: str | list[str] | list[dict[str, Any]] | list[SettingDarrpoptimizeschedulesItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class SettingResponse(TypedDict, total=False):
    """Response type for Setting - use with .dict property for typed dict access."""
    account_id: str
    country: Literal["--", "AF", "AL", "DZ", "AS", "AO", "AR", "AM", "AU", "AT", "AZ", "BS", "BH", "BD", "BB", "BY", "BE", "BZ", "BJ", "BM", "BT", "BO", "BA", "BW", "BR", "BN", "BG", "BF", "KH", "CM", "KY", "CF", "TD", "CL", "CN", "CX", "CO", "CG", "CD", "CR", "HR", "CY", "CZ", "DK", "DJ", "DM", "DO", "EC", "EG", "SV", "ET", "EE", "GF", "PF", "FO", "FJ", "FI", "FR", "GA", "GE", "GM", "DE", "GH", "GI", "GR", "GL", "GD", "GP", "GU", "GT", "GY", "HT", "HN", "HK", "HU", "IS", "IN", "ID", "IQ", "IE", "IM", "IL", "IT", "CI", "JM", "JO", "KZ", "KE", "KR", "KW", "LA", "LV", "LB", "LS", "LR", "LY", "LI", "LT", "LU", "MO", "MK", "MG", "MW", "MY", "MV", "ML", "MT", "MH", "MQ", "MR", "MU", "YT", "MX", "FM", "MD", "MC", "MN", "MA", "MZ", "MM", "NA", "NP", "NL", "AN", "AW", "NZ", "NI", "NE", "NG", "NO", "MP", "OM", "PK", "PW", "PA", "PG", "PY", "PE", "PH", "PL", "PT", "PR", "QA", "RE", "RO", "RU", "RW", "BL", "KN", "LC", "MF", "PM", "VC", "SA", "SN", "RS", "ME", "SL", "SG", "SK", "SI", "SO", "ZA", "ES", "LK", "SR", "SZ", "SE", "CH", "TW", "TZ", "TH", "TL", "TG", "TT", "TN", "TR", "TM", "AE", "TC", "UG", "UA", "GB", "US", "PS", "UY", "UZ", "VU", "VE", "VN", "VI", "WF", "YE", "ZM", "ZW", "JP", "CA"]
    duplicate_ssid: Literal["enable", "disable"]
    fapc_compatibility: Literal["enable", "disable"]
    wfa_compatibility: Literal["enable", "disable"]
    phishing_ssid_detect: Literal["enable", "disable"]
    fake_ssid_action: str
    offending_ssid: list[SettingOffendingssidItem]
    device_weight: int
    device_holdoff: int
    device_idle: int
    firmware_provision_on_authorization: Literal["enable", "disable"]
    rolling_wtp_upgrade: Literal["enable", "disable"]
    darrp_optimize: int
    darrp_optimize_schedules: list[SettingDarrpoptimizeschedulesItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class SettingObject(FortiObject):
    """Typed FortiObject for Setting with field access."""
    account_id: str
    country: Literal["--", "AF", "AL", "DZ", "AS", "AO", "AR", "AM", "AU", "AT", "AZ", "BS", "BH", "BD", "BB", "BY", "BE", "BZ", "BJ", "BM", "BT", "BO", "BA", "BW", "BR", "BN", "BG", "BF", "KH", "CM", "KY", "CF", "TD", "CL", "CN", "CX", "CO", "CG", "CD", "CR", "HR", "CY", "CZ", "DK", "DJ", "DM", "DO", "EC", "EG", "SV", "ET", "EE", "GF", "PF", "FO", "FJ", "FI", "FR", "GA", "GE", "GM", "DE", "GH", "GI", "GR", "GL", "GD", "GP", "GU", "GT", "GY", "HT", "HN", "HK", "HU", "IS", "IN", "ID", "IQ", "IE", "IM", "IL", "IT", "CI", "JM", "JO", "KZ", "KE", "KR", "KW", "LA", "LV", "LB", "LS", "LR", "LY", "LI", "LT", "LU", "MO", "MK", "MG", "MW", "MY", "MV", "ML", "MT", "MH", "MQ", "MR", "MU", "YT", "MX", "FM", "MD", "MC", "MN", "MA", "MZ", "MM", "NA", "NP", "NL", "AN", "AW", "NZ", "NI", "NE", "NG", "NO", "MP", "OM", "PK", "PW", "PA", "PG", "PY", "PE", "PH", "PL", "PT", "PR", "QA", "RE", "RO", "RU", "RW", "BL", "KN", "LC", "MF", "PM", "VC", "SA", "SN", "RS", "ME", "SL", "SG", "SK", "SI", "SO", "ZA", "ES", "LK", "SR", "SZ", "SE", "CH", "TW", "TZ", "TH", "TL", "TG", "TT", "TN", "TR", "TM", "AE", "TC", "UG", "UA", "GB", "US", "PS", "UY", "UZ", "VU", "VE", "VN", "VI", "WF", "YE", "ZM", "ZW", "JP", "CA"]
    duplicate_ssid: Literal["enable", "disable"]
    fapc_compatibility: Literal["enable", "disable"]
    wfa_compatibility: Literal["enable", "disable"]
    phishing_ssid_detect: Literal["enable", "disable"]
    fake_ssid_action: str
    offending_ssid: list[SettingOffendingssidItem]
    device_weight: int
    device_holdoff: int
    device_idle: int
    firmware_provision_on_authorization: Literal["enable", "disable"]
    rolling_wtp_upgrade: Literal["enable", "disable"]
    darrp_optimize: int
    darrp_optimize_schedules: list[SettingDarrpoptimizeschedulesItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class Setting:
    """
    
    Endpoint: wireless_controller/setting
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SettingObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        account_id: str | None = ...,
        country: Literal["--", "AF", "AL", "DZ", "AS", "AO", "AR", "AM", "AU", "AT", "AZ", "BS", "BH", "BD", "BB", "BY", "BE", "BZ", "BJ", "BM", "BT", "BO", "BA", "BW", "BR", "BN", "BG", "BF", "KH", "CM", "KY", "CF", "TD", "CL", "CN", "CX", "CO", "CG", "CD", "CR", "HR", "CY", "CZ", "DK", "DJ", "DM", "DO", "EC", "EG", "SV", "ET", "EE", "GF", "PF", "FO", "FJ", "FI", "FR", "GA", "GE", "GM", "DE", "GH", "GI", "GR", "GL", "GD", "GP", "GU", "GT", "GY", "HT", "HN", "HK", "HU", "IS", "IN", "ID", "IQ", "IE", "IM", "IL", "IT", "CI", "JM", "JO", "KZ", "KE", "KR", "KW", "LA", "LV", "LB", "LS", "LR", "LY", "LI", "LT", "LU", "MO", "MK", "MG", "MW", "MY", "MV", "ML", "MT", "MH", "MQ", "MR", "MU", "YT", "MX", "FM", "MD", "MC", "MN", "MA", "MZ", "MM", "NA", "NP", "NL", "AN", "AW", "NZ", "NI", "NE", "NG", "NO", "MP", "OM", "PK", "PW", "PA", "PG", "PY", "PE", "PH", "PL", "PT", "PR", "QA", "RE", "RO", "RU", "RW", "BL", "KN", "LC", "MF", "PM", "VC", "SA", "SN", "RS", "ME", "SL", "SG", "SK", "SI", "SO", "ZA", "ES", "LK", "SR", "SZ", "SE", "CH", "TW", "TZ", "TH", "TL", "TG", "TT", "TN", "TR", "TM", "AE", "TC", "UG", "UA", "GB", "US", "PS", "UY", "UZ", "VU", "VE", "VN", "VI", "WF", "YE", "ZM", "ZW", "JP", "CA"] | None = ...,
        duplicate_ssid: Literal["enable", "disable"] | None = ...,
        fapc_compatibility: Literal["enable", "disable"] | None = ...,
        wfa_compatibility: Literal["enable", "disable"] | None = ...,
        phishing_ssid_detect: Literal["enable", "disable"] | None = ...,
        fake_ssid_action: str | list[str] | None = ...,
        offending_ssid: str | list[str] | list[dict[str, Any]] | list[SettingOffendingssidItem] | None = ...,
        device_weight: int | None = ...,
        device_holdoff: int | None = ...,
        device_idle: int | None = ...,
        firmware_provision_on_authorization: Literal["enable", "disable"] | None = ...,
        rolling_wtp_upgrade: Literal["enable", "disable"] | None = ...,
        darrp_optimize: int | None = ...,
        darrp_optimize_schedules: str | list[str] | list[dict[str, Any]] | list[SettingDarrpoptimizeschedulesItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SettingObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SettingPayload | None = ...,
        account_id: str | None = ...,
        country: Literal["--", "AF", "AL", "DZ", "AS", "AO", "AR", "AM", "AU", "AT", "AZ", "BS", "BH", "BD", "BB", "BY", "BE", "BZ", "BJ", "BM", "BT", "BO", "BA", "BW", "BR", "BN", "BG", "BF", "KH", "CM", "KY", "CF", "TD", "CL", "CN", "CX", "CO", "CG", "CD", "CR", "HR", "CY", "CZ", "DK", "DJ", "DM", "DO", "EC", "EG", "SV", "ET", "EE", "GF", "PF", "FO", "FJ", "FI", "FR", "GA", "GE", "GM", "DE", "GH", "GI", "GR", "GL", "GD", "GP", "GU", "GT", "GY", "HT", "HN", "HK", "HU", "IS", "IN", "ID", "IQ", "IE", "IM", "IL", "IT", "CI", "JM", "JO", "KZ", "KE", "KR", "KW", "LA", "LV", "LB", "LS", "LR", "LY", "LI", "LT", "LU", "MO", "MK", "MG", "MW", "MY", "MV", "ML", "MT", "MH", "MQ", "MR", "MU", "YT", "MX", "FM", "MD", "MC", "MN", "MA", "MZ", "MM", "NA", "NP", "NL", "AN", "AW", "NZ", "NI", "NE", "NG", "NO", "MP", "OM", "PK", "PW", "PA", "PG", "PY", "PE", "PH", "PL", "PT", "PR", "QA", "RE", "RO", "RU", "RW", "BL", "KN", "LC", "MF", "PM", "VC", "SA", "SN", "RS", "ME", "SL", "SG", "SK", "SI", "SO", "ZA", "ES", "LK", "SR", "SZ", "SE", "CH", "TW", "TZ", "TH", "TL", "TG", "TT", "TN", "TR", "TM", "AE", "TC", "UG", "UA", "GB", "US", "PS", "UY", "UZ", "VU", "VE", "VN", "VI", "WF", "YE", "ZM", "ZW", "JP", "CA"] | None = ...,
        duplicate_ssid: Literal["enable", "disable"] | None = ...,
        fapc_compatibility: Literal["enable", "disable"] | None = ...,
        wfa_compatibility: Literal["enable", "disable"] | None = ...,
        phishing_ssid_detect: Literal["enable", "disable"] | None = ...,
        fake_ssid_action: Literal["log", "suppress"] | list[str] | None = ...,
        offending_ssid: str | list[str] | list[dict[str, Any]] | list[SettingOffendingssidItem] | None = ...,
        device_weight: int | None = ...,
        device_holdoff: int | None = ...,
        device_idle: int | None = ...,
        firmware_provision_on_authorization: Literal["enable", "disable"] | None = ...,
        rolling_wtp_upgrade: Literal["enable", "disable"] | None = ...,
        darrp_optimize: int | None = ...,
        darrp_optimize_schedules: str | list[str] | list[dict[str, Any]] | list[SettingDarrpoptimizeschedulesItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Setting",
    "SettingPayload",
    "SettingResponse",
    "SettingObject",
]


"""
Pydantic Models for CMDB - wireless_controller/ble_profile

Runtime validation models for wireless_controller/ble_profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class BleProfileTxpowerEnum(str, Enum):
    """Allowed values for txpower field."""
    V_0 = "0"
    V_1 = "1"
    V_2 = "2"
    V_3 = "3"
    V_4 = "4"
    V_5 = "5"
    V_6 = "6"
    V_7 = "7"
    V_8 = "8"
    V_9 = "9"
    V_10 = "10"
    V_11 = "11"
    V_12 = "12"
    V_13 = "13"
    V_14 = "14"
    V_15 = "15"
    V_16 = "16"
    V_17 = "17"


# ============================================================================
# Main Model
# ============================================================================

class BleProfileModel(BaseModel):
    """
    Pydantic model for wireless_controller/ble_profile configuration.
    
    Configure Bluetooth Low Energy profile.
    
    Validation Rules:        - name: max_length=35 pattern=        - comment: max_length=63 pattern=        - advertising: pattern=        - ibeacon_uuid: max_length=63 pattern=        - major_id: min=0 max=65535 pattern=        - minor_id: min=0 max=65535 pattern=        - eddystone_namespace: max_length=20 pattern=        - eddystone_instance: max_length=12 pattern=        - eddystone_url: max_length=127 pattern=        - txpower: pattern=        - beacon_interval: min=40 max=3500 pattern=        - ble_scanning: pattern=        - scan_type: pattern=        - scan_threshold: max_length=7 pattern=        - scan_period: min=1000 max=10000 pattern=        - scan_time: min=1000 max=10000 pattern=        - scan_interval: min=10 max=1000 pattern=        - scan_window: min=10 max=1000 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Bluetooth Low Energy profile name.")    
    comment: str | None = Field(max_length=63, default=None, description="Comment.")    
    advertising: list[Literal["ibeacon", "eddystone-uid", "eddystone-url"]] = Field(default_factory=list, description="Advertising type.")    
    ibeacon_uuid: str | None = Field(max_length=63, default="005ea414-cbd1-11e5-9956-625662870761", description="Universally Unique Identifier (UUID; automatically assigned but can be manually reset).")    
    major_id: int | None = Field(ge=0, le=65535, default=1000, description="Major ID.")    
    minor_id: int | None = Field(ge=0, le=65535, default=2000, description="Minor ID.")    
    eddystone_namespace: str | None = Field(max_length=20, default="0102030405", description="Eddystone namespace ID.")    
    eddystone_instance: str | None = Field(max_length=12, default="abcdef", description="Eddystone instance ID.")    
    eddystone_url: str | None = Field(max_length=127, default="http://www.fortinet.com", description="Eddystone URL.")    
    txpower: BleProfileTxpowerEnum | None = Field(default=BleProfileTxpowerEnum.V_0, description="Transmit power level (default = 0).")    
    beacon_interval: int | None = Field(ge=40, le=3500, default=100, description="Beacon interval (default = 100 msec).")    
    ble_scanning: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable Bluetooth Low Energy (BLE) scanning.")    
    scan_type: Literal["active", "passive"] | None = Field(default="active", description="Scan Type (default = active).")    
    scan_threshold: str | None = Field(max_length=7, default="-90", description="Minimum signal level/threshold in dBm required for the AP to report detected BLE device (-95 to -20, default = -90).")    
    scan_period: int | None = Field(ge=1000, le=10000, default=4000, description="Scan Period (default = 4000 msec).")    
    scan_time: int | None = Field(ge=1000, le=10000, default=1000, description="Scan Time (default = 1000 msec).")    
    scan_interval: int | None = Field(ge=10, le=1000, default=50, description="Scan Interval (default = 50 msec).")    
    scan_window: int | None = Field(ge=10, le=1000, default=50, description="Scan Windows (default = 50 msec).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "BleProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "BleProfileModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.399505Z
# ============================================================================


"""
Pydantic Models for CMDB - user/exchange

Runtime validation models for user/exchange configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ExchangeKdcIp(BaseModel):
    """
    Child table model for kdc-ip.
    
    KDC IPv4 addresses for Kerberos authentication.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    ipv4: str = Field(max_length=79, description="KDC IPv4 addresses for Kerberos authentication.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class ExchangeAuthLevelEnum(str, Enum):
    """Allowed values for auth_level field."""
    CONNECT = "connect"
    CALL = "call"
    PACKET = "packet"
    INTEGRITY = "integrity"
    PRIVACY = "privacy"

class ExchangeSslMinProtoVersionEnum(str, Enum):
    """Allowed values for ssl_min_proto_version field."""
    DEFAULT = "default"
    SSLV3 = "SSLv3"
    TLSV1 = "TLSv1"
    TLSV1_1 = "TLSv1-1"
    TLSV1_2 = "TLSv1-2"
    TLSV1_3 = "TLSv1-3"


# ============================================================================
# Main Model
# ============================================================================

class ExchangeModel(BaseModel):
    """
    Pydantic model for user/exchange configuration.
    
    Configure MS Exchange server entries.
    
    Validation Rules:        - name: max_length=35 pattern=        - server_name: max_length=63 pattern=        - domain_name: max_length=79 pattern=        - username: max_length=64 pattern=        - password: max_length=128 pattern=        - ip: pattern=        - connect_protocol: pattern=        - validate_server_certificate: pattern=        - auth_type: pattern=        - auth_level: pattern=        - http_auth_type: pattern=        - ssl_min_proto_version: pattern=        - auto_discover_kdc: pattern=        - kdc_ip: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="MS Exchange server entry name.")    
    server_name: str = Field(max_length=63, description="MS Exchange server hostname.")    
    domain_name: str = Field(max_length=79, description="MS Exchange server fully qualified domain name.")    
    username: str = Field(max_length=64, description="User name used to sign in to the server. Must have proper permissions for service.")    
    password: Any = Field(max_length=128, description="Password for the specified username.")    
    ip: str | None = Field(default="0.0.0.0", description="Server IPv4 address.")    
    connect_protocol: Literal["rpc-over-tcp", "rpc-over-http", "rpc-over-https"] | None = Field(default="rpc-over-https", description="Connection protocol used to connect to MS Exchange service.")    
    validate_server_certificate: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable exchange server certificate validation.")    
    auth_type: Literal["spnego", "ntlm", "kerberos"] | None = Field(default="kerberos", description="Authentication security type used for the RPC protocol layer.")    
    auth_level: ExchangeAuthLevelEnum | None = Field(default=ExchangeAuthLevelEnum.PRIVACY, description="Authentication security level used for the RPC protocol layer.")    
    http_auth_type: Literal["basic", "ntlm"] | None = Field(default="ntlm", description="Authentication security type used for the HTTP transport.")    
    ssl_min_proto_version: ExchangeSslMinProtoVersionEnum | None = Field(default=ExchangeSslMinProtoVersionEnum.DEFAULT, description="Minimum SSL/TLS protocol version for HTTPS transport (default is to follow system global setting).")    
    auto_discover_kdc: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable automatic discovery of KDC IP addresses.")    
    kdc_ip: list[ExchangeKdcIp] = Field(default_factory=list, description="KDC IPv4 addresses for Kerberos authentication.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ExchangeModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ExchangeModel",    "ExchangeKdcIp",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.454087Z
# ============================================================================


"""
Pydantic Models for CMDB - system/mobile_tunnel

Runtime validation models for system/mobile_tunnel configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class MobileTunnelNetwork(BaseModel):
    """
    Child table model for network.
    
    NEMO network configuration.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Network entry ID.")    
    interface: str | None = Field(max_length=15, default=None, description="Select the associated interface name from available options.")  # datasource: ['system.interface.name']    
    prefix: str | None = Field(default="0.0.0.0 0.0.0.0", description="Class IP and Netmask with correction (Format:xxx.xxx.xxx.xxx xxx.xxx.xxx.xxx or xxx.xxx.xxx.xxx/x).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class MobileTunnelModel(BaseModel):
    """
    Pydantic model for system/mobile_tunnel configuration.
    
    Configure Mobile tunnels, an implementation of Network Mobility (NEMO) extensions for Mobile IPv4 RFC5177.
    
    Validation Rules:        - name: max_length=15 pattern=        - status: pattern=        - roaming_interface: max_length=15 pattern=        - home_agent: pattern=        - home_address: pattern=        - renew_interval: min=5 max=60 pattern=        - lifetime: min=180 max=65535 pattern=        - reg_interval: min=5 max=300 pattern=        - reg_retry: min=1 max=30 pattern=        - n_mhae_spi: min=0 max=4294967295 pattern=        - n_mhae_key_type: pattern=        - n_mhae_key: pattern=        - hash_algorithm: pattern=        - tunnel_mode: pattern=        - network: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=15, default=None, description="Tunnel name.")    
    status: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable this mobile tunnel.")    
    roaming_interface: str = Field(max_length=15, description="Select the associated interface name from available options.")  # datasource: ['system.interface.name']    
    home_agent: str = Field(default="0.0.0.0", description="IPv4 address of the NEMO HA (Format: xxx.xxx.xxx.xxx).")    
    home_address: str | None = Field(default="0.0.0.0", description="Home IP address (Format: xxx.xxx.xxx.xxx).")    
    renew_interval: int = Field(ge=5, le=60, default=60, description="Time before lifetime expiration to send NMMO HA re-registration (5 - 60, default = 60).")    
    lifetime: int = Field(ge=180, le=65535, default=65535, description="NMMO HA registration request lifetime (180 - 65535 sec, default = 65535).")    
    reg_interval: int = Field(ge=5, le=300, default=5, description="NMMO HA registration interval (5 - 300, default = 5).")    
    reg_retry: int = Field(ge=1, le=30, default=3, description="Maximum number of NMMO HA registration retries (1 to 30, default = 3).")    
    n_mhae_spi: int = Field(ge=0, le=4294967295, default=256, description="NEMO authentication SPI (default: 256).")    
    n_mhae_key_type: Literal["ascii", "base64"] = Field(default="ascii", description="NEMO authentication key type (ASCII or base64).")    
    n_mhae_key: Any = Field(default=None, description="NEMO authentication key.")    
    hash_algorithm: Literal["hmac-md5"] = Field(default="hmac-md5", description="Hash Algorithm (Keyed MD5).")    
    tunnel_mode: Literal["gre"] = Field(default="gre", description="NEMO tunnel mode (GRE tunnel).")    
    network: list[MobileTunnelNetwork] = Field(default_factory=list, description="NEMO network configuration.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('roaming_interface')
    @classmethod
    def validate_roaming_interface(cls, v: Any) -> Any:
        """
        Validate roaming_interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "MobileTunnelModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_roaming_interface_references(self, client: Any) -> list[str]:
        """
        Validate roaming_interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = MobileTunnelModel(
            ...     roaming_interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_roaming_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.mobile_tunnel.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "roaming_interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Roaming-Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_network_references(self, client: Any) -> list[str]:
        """
        Validate network references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = MobileTunnelModel(
            ...     network=[{"interface": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_network_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.system.mobile_tunnel.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "network", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("interface")
            else:
                value = getattr(item, "interface", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Network '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_roaming_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_network_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "MobileTunnelModel",    "MobileTunnelNetwork",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.699585Z
# ============================================================================


"""
Pydantic Models for CMDB - switch_controller/global_

Runtime validation models for switch_controller/global_ configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class GlobalDisableDiscovery(BaseModel):
    """
    Child table model for disable-discovery.
    
    Prevent this FortiSwitch from discovering.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=79, default=None, description="FortiSwitch Serial-number.")
class GlobalCustomCommand(BaseModel):
    """
    Child table model for custom-command.
    
    List of custom commands to be pushed to all FortiSwitches in the VDOM.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    command_entry: str | None = Field(max_length=35, default=None, description="List of FortiSwitch commands.")    
    command_name: str = Field(max_length=35, description="Name of custom command to push to all FortiSwitches in VDOM.")  # datasource: ['switch-controller.custom-command.command-name']
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class GlobalDhcpOption82CircuitIdEnum(str, Enum):
    """Allowed values for dhcp_option82_circuit_id field."""
    INTFNAME = "intfname"
    VLAN = "vlan"
    HOSTNAME = "hostname"
    MODE = "mode"
    DESCRIPTION = "description"

class GlobalUpdateUserDeviceEnum(str, Enum):
    """Allowed values for update_user_device field."""
    MAC_CACHE = "mac-cache"
    LLDP = "lldp"
    DHCP_SNOOPING = "dhcp-snooping"
    L2_DB = "l2-db"
    L3_DB = "l3-db"


# ============================================================================
# Main Model
# ============================================================================

class GlobalModel(BaseModel):
    """
    Pydantic model for switch_controller/global_ configuration.
    
    Configure FortiSwitch global settings.
    
    Validation Rules:        - mac_aging_interval: min=10 max=1000000 pattern=        - https_image_push: pattern=        - vlan_all_mode: pattern=        - vlan_optimization: pattern=        - vlan_identity: pattern=        - disable_discovery: pattern=        - mac_retention_period: min=0 max=168 pattern=        - default_virtual_switch_vlan: max_length=15 pattern=        - dhcp_server_access_list: pattern=        - dhcp_option82_format: pattern=        - dhcp_option82_circuit_id: pattern=        - dhcp_option82_remote_id: pattern=        - dhcp_snoop_client_req: pattern=        - dhcp_snoop_client_db_exp: min=300 max=259200 pattern=        - dhcp_snoop_db_per_port_learn_limit: min=0 max=2048 pattern=        - log_mac_limit_violations: pattern=        - mac_violation_timer: min=0 max=4294967295 pattern=        - sn_dns_resolution: pattern=        - mac_event_logging: pattern=        - bounce_quarantined_link: pattern=        - quarantine_mode: pattern=        - update_user_device: pattern=        - custom_command: pattern=        - fips_enforce: pattern=        - firmware_provision_on_authorization: pattern=        - switch_on_deauth: pattern=        - firewall_auth_user_hold_period: min=5 max=1440 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    mac_aging_interval: int | None = Field(ge=10, le=1000000, default=300, description="Time after which an inactive MAC is aged out (10 - 1000000 sec, default = 300, 0 = disable).")    
    https_image_push: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable image push to FortiSwitch using HTTPS.")    
    vlan_all_mode: Literal["all", "defined"] | None = Field(default="defined", description="VLAN configuration mode, user-defined-vlans or all-possible-vlans.")    
    vlan_optimization: Literal["prune", "configured", "none"] | None = Field(default="configured", description="FortiLink VLAN optimization.")    
    vlan_identity: Literal["description", "name"] | None = Field(default="name", description="Identity of the VLAN. Commonly used for RADIUS Tunnel-Private-Group-Id.")    
    disable_discovery: list[GlobalDisableDiscovery] = Field(default_factory=list, description="Prevent this FortiSwitch from discovering.")    
    mac_retention_period: int | None = Field(ge=0, le=168, default=24, description="Time in hours after which an inactive MAC is removed from client DB (0 = aged out based on mac-aging-interval).")    
    default_virtual_switch_vlan: str | None = Field(max_length=15, default=None, description="Default VLAN for ports when added to the virtual-switch.")  # datasource: ['system.interface.name']    
    dhcp_server_access_list: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable DHCP snooping server access list.")    
    dhcp_option82_format: Literal["ascii", "legacy"] | None = Field(default="ascii", description="DHCP option-82 format string.")    
    dhcp_option82_circuit_id: list[GlobalDhcpOption82CircuitIdEnum] = Field(default_factory=list, description="List the parameters to be included to inform about client identification.")    
    dhcp_option82_remote_id: list[Literal["mac", "hostname", "ip"]] = Field(default_factory=list, description="List the parameters to be included to inform about client identification.")    
    dhcp_snoop_client_req: Literal["drop-untrusted", "forward-untrusted"] | None = Field(default="drop-untrusted", description="Client DHCP packet broadcast mode.")    
    dhcp_snoop_client_db_exp: int | None = Field(ge=300, le=259200, default=86400, description="Expiry time for DHCP snooping server database entries (300 - 259200 sec, default = 86400 sec).")    
    dhcp_snoop_db_per_port_learn_limit: int | None = Field(ge=0, le=2048, default=64, description="Per Interface dhcp-server entries learn limit (0 - 1024, default = 64).")    
    log_mac_limit_violations: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable logs for Learning Limit Violations.")    
    mac_violation_timer: int | None = Field(ge=0, le=4294967295, default=0, description="Set timeout for Learning Limit Violations (0 = disabled).")    
    sn_dns_resolution: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable DNS resolution of the FortiSwitch unit's IP address with switch name.")    
    mac_event_logging: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable MAC address event logging.")    
    bounce_quarantined_link: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable bouncing (administratively bring the link down, up) of a switch port where a quarantined device was seen last. Helps to re-initiate the DHCP process for a device.")    
    quarantine_mode: Literal["by-vlan", "by-redirect"] | None = Field(default="by-vlan", description="Quarantine mode.")    
    update_user_device: list[GlobalUpdateUserDeviceEnum] = Field(default_factory=list, description="Control which sources update the device user list.")    
    custom_command: list[GlobalCustomCommand] = Field(default_factory=list, description="List of custom commands to be pushed to all FortiSwitches in the VDOM.")    
    fips_enforce: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable enforcement of FIPS on managed FortiSwitch devices.")    
    firmware_provision_on_authorization: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable automatic provisioning of latest firmware on authorization.")    
    switch_on_deauth: Literal["no-op", "factory-reset"] | None = Field(default="no-op", description="No-operation/Factory-reset the managed FortiSwitch on deauthorization.")    
    firewall_auth_user_hold_period: int | None = Field(ge=5, le=1440, default=5, description="Time period in minutes to hold firewall authenticated MAC users (5 - 1440, default = 5, disable = 0).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('default_virtual_switch_vlan')
    @classmethod
    def validate_default_virtual_switch_vlan(cls, v: Any) -> Any:
        """
        Validate default_virtual_switch_vlan field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "GlobalModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_default_virtual_switch_vlan_references(self, client: Any) -> list[str]:
        """
        Validate default_virtual_switch_vlan references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     default_virtual_switch_vlan="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_default_virtual_switch_vlan_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.global_.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "default_virtual_switch_vlan", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Default-Virtual-Switch-Vlan '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_custom_command_references(self, client: Any) -> list[str]:
        """
        Validate custom_command references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/custom-command        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = GlobalModel(
            ...     custom_command=[{"command-name": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_custom_command_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.global_.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "custom_command", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("command-name")
            else:
                value = getattr(item, "command-name", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.switch_controller.custom_command.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Custom-Command '{value}' not found in "
                    "switch-controller/custom-command"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_default_virtual_switch_vlan_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_custom_command_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "GlobalModel",    "GlobalDisableDiscovery",    "GlobalCustomCommand",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.108914Z
# ============================================================================